@@ -12,7 +12,6 @@ import (
 	"strings"
 	"testing"
 
-	"github.com/VividCortex/mysqlerr"
 	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
 	"github.com/skeema/skeema/internal/shellout"
@@ -368,14 +367,14 @@ func (dir *Dir) FirstInstance() (*tengo.Instance, error) {
 func (dir *Dir) ValidateInstance(instance *tengo.Instance) error {
 	ok, err := instance.Valid()
 	if !ok {
-		if instance.Password == "" && tengo.IsDatabaseError(err, mysqlerr.ER_ACCESS_DENIED_ERROR) {
+		if instance.Password == "" && tengo.IsAccessDeniedError(err) {
 			err = fmt.Errorf("%w\nNo password was supplied for this login attempt, but the server likely requires a password. For information on how to use Skeema's password option, see https://www.skeema.io/docs/options/#password", err)
 		} else if dir.Config.Changed("connect-options") {
-			if tengo.IsDatabaseError(err, mysqlerr.ER_SPECIFIC_ACCESS_DENIED_ERROR) {
+			if tengo.IsAccessPrivilegeError(err) {
 				err = fmt.Errorf("%w\nCheck your Skeema configuration for connect-options. Typically this error means one of your session variables requires privileges that your user lacks.\nFor more information, see https://www.skeema.io/docs/options/#connect-options", err)
-			} else if tengo.IsDatabaseError(err, mysqlerr.ER_UNKNOWN_SYSTEM_VARIABLE, mysqlerr.ER_INCORRECT_GLOBAL_LOCAL_VAR) {
+			} else if tengo.IsSessionVarNameError(err) {
 				err = fmt.Errorf("%w\nCheck your Skeema configuration for connect-options. Typically this error means one of your session variable names has a typo, or the variable does not exist at the session scope in your specific server version.", err)
-			} else if tengo.IsDatabaseError(err, mysqlerr.ER_WRONG_VALUE_FOR_VAR, mysqlerr.ER_WRONG_TYPE_FOR_VAR) {
+			} else if tengo.IsSessionVarValueError(err) {
 				err = fmt.Errorf("%w\nCheck your Skeema configuration for connect-options. Typically this error means one of your session variable values has a typo, or the value is not supported in your specific server version.", err)
 			}
 		}
@@ -389,7 +388,7 @@ func (dir *Dir) ValidateInstance(instance *tengo.Instance) error {
 			log.Warnf("Instance %s actual flavor %s differs from dir %s configured flavor %s\nIf you have recently upgraded your database server version, consider using `skeema pull` to update your schema definitions.", instance, instFlavor, dir, confFlavor)
 		}
 		if instFlavor.TooNew() {
-			log.Warnf("Instance %s flavor %s is newer than this version of Skeema. To ensure correct behavior, consider upgrading to a newer release of Skeema.", instance, instFlavor)
+			log.Warnf("Instance %s flavor %s is newer than this version of Skeema. To ensure correct behavior, consider upgrading to a more recent release of Skeema.", instance, instFlavor)
 		}
 	} else if confFlavor.Known() {
 		log.Debugf("Instance %s flavor cannot be parsed; using dir %s configured flavor %s instead", instance, dir, confFlavor)
