@@ -3,7 +3,6 @@ package main
 import (
 	"errors"
 	"fmt"
-	log "github.com/sirupsen/logrus"
 	"os"
 	"path/filepath"
 	"reflect"
@@ -12,10 +11,12 @@ import (
 	"strings"
 	"testing"
 
+	log "github.com/sirupsen/logrus"
 	"github.com/skeema/mybase"
 	"github.com/skeema/skeema/internal/fs"
 	"github.com/skeema/skeema/internal/tengo"
 	"github.com/skeema/skeema/internal/util"
+	"github.com/skeema/skeema/internal/workspace"
 )
 
 func TestMain(m *testing.M) {
@@ -190,11 +191,11 @@ func (s *SkeemaIntegrationSuite) verifyFiles(t *testing.T, cfg *mybase.Config, d
 	// CURRENT_TIMESTAMP does not take an arg for specifying sub-second precision
 	// In MySQL 8.0+, partitions are formatted differently; the default character
 	// set is now utf8mb4; the default collation for utf8mb4 has also changed.
-	if s.d.Flavor().Min(tengo.FlavorMariaDB102) {
+	if s.d.Flavor().MinMariaDB(10, 2) {
 		dirExpectedBase = strings.Replace(dirExpectedBase, "golden", "golden-mariadb102", 1)
-	} else if s.d.Flavor().Matches(tengo.FlavorMySQL55) {
+	} else if s.d.Flavor().IsMySQL(5, 5) {
 		dirExpectedBase = strings.Replace(dirExpectedBase, "golden", "golden-mysql55", 1)
-	} else if s.d.Flavor().Min(tengo.FlavorMySQL80) {
+	} else if s.d.Flavor().MinMySQL(8) {
 		dirExpectedBase = strings.Replace(dirExpectedBase, "golden", "golden-mysql80", 1)
 	}
 
@@ -291,10 +292,9 @@ func (s *SkeemaIntegrationSuite) compareDirOptionFiles(t *testing.T, a, b *fs.Di
 				} else if fileCharSet == "utf8" {
 					// MySQL 8.0.29 uses "utf8mb3" but keeps collations as-is; MySQL 8.0.30+
 					// and MariaDB 10.6+ use "utf8mb3" and also changes collation names to match
-					mysql8029 := tengo.FlavorMySQL80.Dot(29)
-					if flavor := s.d.Flavor(); flavor.Min(mysql8029) || flavor.Min(tengo.FlavorMariaDB106) {
+					if flavor := s.d.Flavor(); flavor.MinMySQL(8, 0, 29) || flavor.MinMariaDB(10, 6) {
 						a.OptionFile.SetOptionValue(section, "default-character-set", "utf8mb3")
-						if !flavor.Matches(mysql8029) {
+						if !flavor.IsMySQL(8, 0, 29) {
 							collation, _ := a.OptionFile.OptionValue("default-collation")
 							a.OptionFile.SetOptionValue(section, "default-collation", strings.Replace(collation, "utf8_", "utf8mb3_", 1))
 						}
@@ -486,3 +486,22 @@ func getOptionFile(t *testing.T, basePath string, baseConfig *mybase.Config) *my
 	}
 	return dir.OptionFile
 }
+
+// imageForFlavor returns a Docker image name corresponding to a supplied
+// tengo.Flavor, reusing some logic from the workspace subpackage in order
+// to appropriately handle Percona Server image selection on arm64. The exact
+// patch number of the supplied flavor is ignored in favor of selecting the
+// latest patch of that release series.
+// If a corresponding image cannot be selected, the test fails.
+func imageForFlavor(t *testing.T, flavor tengo.Flavor) string {
+	t.Helper()
+	arch, err := tengo.DockerEngineArchitecture()
+	if err != nil {
+		t.Fatalf("Unable to determine Docker Engine architecture: %v", err)
+	}
+	image, err := workspace.DockerImageForFlavor(flavor.Family(), arch)
+	if err != nil {
+		t.Fatalf("Unable to locate a Docker image corresponding to flavor %s: %v", flavor.Family(), err)
+	}
+	return image
+}
